#
# Author:: Ranjib Dey (<ranjib@pagerduty.com>)
# Copyright:: Copyright (c) 2014 PagerDuty, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'serfx'
require 'blender/log'

module Blender
  module Discovery
    class Serf
      Property = Struct.new(:key, :hook)
      attr_reader :attribute

      def initialize(opts = {})
        @config = {}
        @config[:host] = opts[:host] if opts.key?(:host)
        @config[:port] = opts[:port] if opts.key?(:port)
        @config[:authkey] = opts[:authkey] if opts.key?(:authkey)
        @attribute = opts[:attribute] || 'name'
      end

      def search(opts = {})
        opts = {} if opts.nil?
        tags = opts[:tags] || {}
        status = opts[:status] || 'alive'
        name = opts[:name]
        hosts = []
        Blender::Log.debug("Using Serf agents '#{attribute}' as discovery return values")
        case attribute
        when 'name'
          property = Property.new('Name', lambda{|data| data})
        when 'ipaddress'
          property = Property.new('Addr', lambda{|data| data.unpack('CCCC').join('.')})
        else
          raise ArgumentError, "Attribute can be either 'name' or 'ipaddress', you have provided '#{attribute}'"
        end
        Blender::Log.debug("Serf memeber list call with arguments: Tags=#{tags}")
        Blender::Log.debug("Serf memeber list call with arguments: status=#{status}")
        Blender::Log.debug("Serf memeber list call with arguments: Name=#{name}")
        Serfx.connect(@config) do |conn|
          conn.members_filtered(tags, status, name).body['Members'].map do |m|
            hosts << property.hook.call(m[property.key])
          end
        end
        hosts
      end
    end
  end
end
