#
# Author:: Ranjib Dey (<ranjib@pagerduty.com>)
# Copyright:: Copyright (c) 2014 PagerDuty, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'blender/exceptions'
require 'blender/log'
require 'blender/drivers/serf'
require 'json'
require 'blender/tasks/serf'
require 'retriable'

module Blender
  module Driver
    class SerfAsync < Blender::Driver::Base
      def initialize(config = {})
        super
        @driver = Blender::Driver::Serf.new(config)
      end

      def execute(tasks, hosts)
        Log.debug("Serf Async query on #{@driver.filter_by}s [#{hosts.inspect}]")
        tasks.each do |task|
          retry_options = task.metadata[:retry_options]
          hosts.each_slice(@driver.concurrency) do |nodes|
            Blender::Log.debug("Start query: #{task.start_query.inspect}")
            start = @driver.run_command(task.start_query.command, nodes)
            if start.exitstatus != 0 and !task.metadata[:ignore_failure]
              raise ExecutionFailed, start.stderr
            end
            Blender::Log.debug("Using check retry options:#{retry_options}")
            Retriable.retriable(retry_options) do
              cmd = @driver.run_command(task.check_query.command, nodes)
              if cmd.exitstatus != 0 and !task.metadata[:ignore_failure]
                raise ExecutionFailed, cmd.stderr
              end
            end
            cmd = @driver.run_command(task.reap_query.command, nodes)
            if cmd.exitstatus != 0 and !task.metadata[:ignore_failure]
              raise ExecutionFailed, cmd.stderr
            end
          end
        end
      end
    end
  end
end
