#
# Author:: Ranjib Dey (<ranjib@pagerduty.com>)
# Copyright:: Copyright (c) 2015 PagerDuty, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
require 'blender/tasks/serf'

module Blender
  module Task
    class SerfAsync < Blender::Task::Base
      attr_reader :start_query, :reap_query, :check_query

      def initialize(name, options = {})
        super
        @start_query = Blender::Task::Serf.new(name)
        @start_query.payload(metadata[:start])
        @start_query.timeout(metadata[:timeout])
        @start_query.no_ack(metadata[:no_ack])
        @start_query.process(&metadata[:process])

        @reap_query = Blender::Task::Serf.new(name)
        @reap_query.payload(metadata[:reap])
        @reap_query.timeout(metadata[:timeout])
        @reap_query.no_ack(metadata[:no_ack])
        @reap_query.process(&metadata[:process])

        @check_query = Blender::Task::Serf.new(name)
        @check_query.payload(metadata[:check])
        @check_query.timeout(metadata[:timeout])
        @check_query.no_ack(metadata[:no_ack])
        @check_query.process(&metadata[:process])
      end

      def query(event)
        @start_query.query(event)
        @reap_query.query(event)
        @check_query.query(event)
      end

      def start(&block)
        @start_query.instance_eval(&block)
      end

      def reap(&block)
        @reap_query.instance_eval(&block)
      end

      def check(&block)
        @check_query.instance_eval(&block)
      end

      def execute
      end

      def command
      end

      def retry_options(options = {})
        @metadata[:retry_options] = options
      end

      def default_metadata
        {
          ignore_failure: false,
          start: 'start',
          reap: 'reap',
          check: 'check',
          timeout: 5,
          no_ack: false,
          process: nil,
          retry_options: { intervals: Array.new(5, 30) }
        }
      end
    end
  end
end
